@@ -141,17 +141,20 @@ class MaskedConstant(numpy.ma.core.MaskedConstant):
     """A trivial extension of numpy.ma.masked
 
     We want to be able to put the generic term ``masked`` into a dictionary.
-    In python 2.7 we can just use ``numpy.ma.masked``, but in python 3.1 and 3.2 that
-    is not hashable, see https://github.com/numpy/numpy/issues/4660
-    So, we need to extend it here with a hash value.
+    The constant ``numpy.ma.masked`` is not hashable (see
+    https://github.com/numpy/numpy/issues/4660), so we need to extend it
+    here with a hash value.
     """
+
     def __hash__(self):
         '''All instances of this class shall have the same hash.'''
         # Any large number will do.
         return 1234567890
 
+
 masked = MaskedConstant()
 
+
 class InconsistentTableError(ValueError):
     """
     Indicates that an input table is inconsistent in some way.
@@ -160,6 +163,7 @@ class InconsistentTableError(ValueError):
     this class if a data row doesn't match the header.
     """
 
+
 class OptionalTableImportError(ImportError):
     """
     Indicates that a dependency for table reading is not present.
@@ -169,6 +173,7 @@ class OptionalTableImportError(ImportError):
     an ImportError.
     """
 
+
 class ParameterError(NotImplementedError):
     """
     Indicates that a reader cannot handle a passed parameter.
@@ -178,12 +183,14 @@ class ParameterError(NotImplementedError):
     C engine cannot handle.
     """
 
+
 class FastOptionsError(NotImplementedError):
     """
     Indicates that one of the specified options for fast
     reading is invalid.
     """
 
+
 class NoType(object):
     """
     Superclass for ``StrType`` and ``NumType`` classes.
@@ -243,6 +250,7 @@ class Column(object):
     * **str_vals** : list of column values as strings
     * **data** : list of converted column values
     """
+
     def __init__(self, name):
         self.name = name
         self.type = NoType  # Generic type (Int, Float, Str etc)
@@ -256,6 +264,10 @@ class BaseInputter(object):
     Get the lines from the table input and return a list of lines.
 
     """
+
+    encoding = None
+    """Encoding used to read the file"""
+
     def get_lines(self, table):
         """
         Get the lines from the ``table`` input. The input table can be one of:
@@ -280,8 +292,9 @@ class BaseInputter(object):
         try:
             if (hasattr(table, 'read') or
                     ('\n' not in table + '' and '\r' not in table + '')):
-                with get_readable_fileobj(table) as file_obj:
-                    table = file_obj.read()
+                with get_readable_fileobj(table,
+                                          encoding=self.encoding) as fileobj:
+                    table = fileobj.read()
             lines = table.splitlines()
         except TypeError:
             try:
@@ -387,7 +400,6 @@ class DefaultSplitter(BaseSplitter):
     csv_writer = None
     csv_writer_out = StringIO()
 
-
     def process_line(self, line):
         """Remove whitespace at the beginning or end of line.  This is especially useful for
         whitespace-delimited files to prevent spurious columns at the beginning or end.
@@ -396,7 +408,6 @@ class DefaultSplitter(BaseSplitter):
             line = _replace_tab_with_space(line, self.escapechar, self.quotechar)
         return line.strip()
 
-
     def __call__(self, lines):
         """Return an iterator over the table ``lines``, where each iterator output
         is a list of the split line values.
@@ -972,6 +983,8 @@ class TableOutputter(BaseOutputter):
                           convert_numpy(numpy.str)]
 
     def __call__(self, cols, meta):
+        # Sets col.data to numpy array and col.type to io.ascii Type class (e.g.
+        # FloatType) for each col.
         self._convert_vals(cols)
 
         # If there are any values that were filled and tagged with a mask bit then this
@@ -1027,6 +1040,7 @@ def _is_number(x):
         return True
     return False
 
+
 def _apply_include_exclude_names(table, names, include_names, exclude_names):
     """
     Apply names, include_names and exclude_names to a table.
@@ -1085,6 +1099,7 @@ class BaseReader(object):
     exclude_names = None
     strict_names = False
     guessing = False
+    encoding = None
 
     header_class = BaseHeader
     data_class = BaseData
@@ -1182,9 +1197,9 @@ class BaseReader(object):
                 col.str_vals.append(str_vals[j])
 
         self.data.masks(cols)
-        table = self.outputter(cols, self.meta)
         if hasattr(self.header, 'table_meta'):
-            table.meta.update(self.header.table_meta)
+            self.meta['table'].update(self.header.table_meta)
+        table = self.outputter(cols, self.meta)
         self.cols = self.header.cols
 
         _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names)
@@ -1231,6 +1246,26 @@ class BaseReader(object):
             comment_lines = []
         return comment_lines
 
+    def update_table_data(self, table):
+        """
+        Update table columns in place if needed.
+
+        This is a hook to allow updating the table columns after name
+        filtering but before setting up to write the data.  This is currently
+        only used by ECSV and is otherwise just a pass-through.
+
+        Parameters
+        ----------
+        table : `astropy.table.Table`
+            Input table for writing
+
+        Returns
+        -------
+        table : `astropy.table.Table`
+            Output table for writing
+        """
+        return table
+
     def write_header(self, lines, meta):
         self.header.write_comments(lines, meta)
         self.header.write(lines)
@@ -1255,8 +1290,16 @@ class BaseReader(object):
         self.header.cols = list(six.itervalues(table.columns))
         self.header.check_column_names(self.names, self.strict_names, False)
 
+        # In-place update of columns in input ``table`` to reflect column
+        # filtering.  Note that ``table`` is guaranteed to be a copy of the
+        # original user-supplied table.
         _apply_include_exclude_names(table, self.names, self.include_names, self.exclude_names)
 
+        # This is a hook to allow updating the table columns after name
+        # filtering but before setting up to write the data.  This is currently
+        # only used by ECSV and is otherwise just a pass-through.
+        table = self.update_table_data(table)
+
         # Now use altered columns
         new_cols = list(six.itervalues(table.columns))
         # link information about the columns to the writer object (i.e. self)
@@ -1325,9 +1368,10 @@ class WhitespaceSplitter(DefaultSplitter):
 
         return ''.join(newline)
 
+
 extra_reader_pars = ('Reader', 'Inputter', 'Outputter',
                      'delimiter', 'comment', 'quotechar', 'header_start',
-                     'data_start', 'data_end', 'converters',
+                     'data_start', 'data_end', 'converters', 'encoding',
                      'data_Splitter', 'header_Splitter',
                      'names', 'include_names', 'exclude_names', 'strict_names',
                      'fill_values', 'fill_include_names', 'fill_exclude_names')
@@ -1340,13 +1384,13 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
     """
 
     from .fastbasic import FastBasic
-    if issubclass(Reader, FastBasic): # Fast readers handle args separately
+    if issubclass(Reader, FastBasic):  # Fast readers handle args separately
         if Inputter is not None:
             kwargs['Inputter'] = Inputter
         return Reader(**kwargs)
 
     if 'fast_reader' in kwargs:
-        del kwargs['fast_reader'] # ignore fast_reader parameter for slow readers
+        del kwargs['fast_reader']  # ignore fast_reader parameter for slow readers
     reader_kwargs = dict([k, v] for k, v in kwargs.items() if k not in extra_reader_pars)
     reader = Reader(**reader_kwargs)
 
@@ -1412,9 +1456,17 @@ def _get_reader(Reader, Inputter=None, Outputter=None, **kwargs):
         reader.data.fill_include_names = kwargs['fill_include_names']
     if 'fill_exclude_names' in kwargs:
         reader.data.fill_exclude_names = kwargs['fill_exclude_names']
+    if 'encoding' in kwargs:
+        if six.PY2:
+            raise ValueError("the encoding parameter is not supported on "
+                             "Python 2")
+        else:
+            reader.encoding = kwargs['encoding']
+            reader.inputter.encoding = kwargs['encoding']
 
     return reader
 
+
 extra_writer_pars = ('delimiter', 'comment', 'quotechar', 'formats',
                      'strip_whitespace',
                      'names', 'include_names', 'exclude_names',
@@ -1436,7 +1488,7 @@ def _get_writer(Writer, fast_writer, **kwargs):
     if 'fill_values' in kwargs and kwargs['fill_values'] is None:
         del kwargs['fill_values']
 
-    if issubclass(Writer, FastBasic): # Fast writers handle args separately
+    if issubclass(Writer, FastBasic):  # Fast writers handle args separately
         return Writer(**kwargs)
     elif fast_writer and 'fast_{0}'.format(Writer._format_name) in FAST_CLASSES:
         # Switch to fast writer
