@@ -80,7 +80,7 @@ class IpacHeader(fixedwidth.FixedWidthHeader):
                      ('real', core.FloatType),
                      ('char', core.StrType),
                      ('date', core.StrType))
-    definition='ignore'
+    definition = 'ignore'
     start_line = None
 
     def process_lines(self, lines):
@@ -217,8 +217,6 @@ class IpacHeader(fixedwidth.FixedWidthHeader):
         self.names = [x.name for x in cols]
         self.cols = cols
 
-
-
     def str_vals(self):
 
         if self.DBMS:
@@ -278,7 +276,7 @@ class IpacHeader(fixedwidth.FixedWidthHeader):
             null = col.fill_values[core.masked]
             try:
                 format_key = (col_format, col.info.name)
-                auto_format_func = get_auto_format_func(col.info.name)
+                auto_format_func = get_auto_format_func(id(col))
                 format_func = _format_funcs.get(format_key, auto_format_func)
                 nullist.append((format_func(col_format, null)).strip())
             except Exception:
@@ -492,7 +490,7 @@ class Ipac(basic.Basic):
                          "IPAC metadata must be in the form {{'keywords':"
                          "{{'keyword': {{'value': value}} }}".format(keyword),
                          AstropyUserWarning)
-        ignored_keys = [key for key in table.meta if key not in ('keywords','comments')]
+        ignored_keys = [key for key in table.meta if key not in ('keywords', 'comments')]
         if any(ignored_keys):
             warn("Table metadata keyword(s) {0} were not written.  "
                  "IPAC metadata must be in the form {{'keywords':"
