@@ -10,6 +10,7 @@ from .common import (assert_equal, assert_almost_equal,
 def assert_equal_splitlines(arg1, arg2):
     assert_equal(arg1.splitlines(), arg2.splitlines())
 
+
 def test_read_normal():
     """Normal SimpleRST Table"""
     table = """
@@ -28,6 +29,7 @@ def test_read_normal():
     assert_equal(dat[0][1], '"hello"')
     assert_equal(dat[1][1], "'s worlds")
 
+
 def test_read_normal_names():
     """Normal SimpleRST Table with provided column names"""
     table = """
@@ -45,6 +47,7 @@ def test_read_normal_names():
     assert_equal(dat.colnames, ['name1', 'name2'])
     assert_almost_equal(dat[1][0], 2.4)
 
+
 def test_read_normal_names_include():
     """Normal SimpleRST Table with provided column names"""
     table = """
@@ -64,6 +67,7 @@ def test_read_normal_names_include():
     assert_almost_equal(dat[1][0], 2.4)
     assert_equal(dat[0][1], 3)
 
+
 def test_read_normal_exclude():
     """Nice, typical SimpleRST table with col name excluded"""
     table = """
@@ -80,6 +84,7 @@ def test_read_normal_exclude():
     assert_equal(dat.colnames, ['Col2'])
     assert_equal(dat[1][0], "'s worlds")
 
+
 def test_read_unbounded_right_column():
     """The right hand column should be allowed to overflow"""
     table = """
@@ -96,6 +101,7 @@ def test_read_unbounded_right_column():
     assert_equal(dat[0][2], "Hello")
     assert_equal(dat[1][2], "Worlds")
 
+
 def test_read_unbounded_right_column_header():
     """The right hand column should be allowed to overflow"""
     table = """
@@ -111,6 +117,7 @@ def test_read_unbounded_right_column_header():
     dat = reader.read(table)
     assert_equal(dat.colnames[-1], "Col3Long")
 
+
 def test_read_right_indented_table():
     """We should be able to read right indented tables correctly"""
     table = """
@@ -128,6 +135,7 @@ def test_read_right_indented_table():
     assert_equal(dat[0][2], "foo")
     assert_equal(dat[1][0], 1)
 
+
 def test_trailing_spaces_in_row_definition():
     """ Trailing spaces in the row definition column shouldn't matter"""
     table = """
@@ -138,7 +146,7 @@ def test_trailing_spaces_in_row_definition():
     3    3.4  foo
     1    4.5  bar
    ==== ==== ====  
-"""  # nopep8
+"""  # noqa
     reader = ascii.get_reader(Reader=ascii.RST)
     dat = reader.read(table)
     assert_equal(dat.colnames, ["Col1", "Col2", "Col3"])
@@ -156,6 +164,7 @@ table = """\
 """
 dat = ascii.read(table, Reader=ascii.RST)
 
+
 def test_write_normal():
     """Write a table as a normal SimpleRST Table"""
     out = StringIO()
