@@ -74,12 +74,13 @@ except ImportError:
 else:
     HAS_PATHLIB = True
 
+
 class _File(object):
     """
     Represents a FITS file on disk (or in some other file-like object).
     """
 
-    @deprecated_renamed_argument('clobber', 'overwrite', '1.3')
+    @deprecated_renamed_argument('clobber', 'overwrite', '2.0')
     def __init__(self, fileobj=None, mode=None, memmap=None, overwrite=False,
                  cache=True):
         self.strict_memmap = bool(memmap)
@@ -95,6 +96,7 @@ class _File(object):
             self.readonly = False
             self.writeonly = False
             self.simulateonly = True
+            self.close_on_error = False
             return
         else:
             self.simulateonly = False
@@ -119,7 +121,7 @@ class _File(object):
 
         if (isinstance(fileobj, string_types) and
             mode not in ('ostream', 'append') and
-            _is_url(fileobj)): # This is an URL.
+            _is_url(fileobj)):  # This is an URL.
             self.name = download_file(fileobj, cache=cache)
         else:
             self.name = fileobj_name(fileobj)
@@ -132,6 +134,10 @@ class _File(object):
         # Underlying fileobj is a file-like object, but an actual file object
         self.file_like = False
 
+        # Should the object be closed on error: see
+        # https://github.com/astropy/astropy/issues/6168
+        self.close_on_error = False
+
         # More defaults to be adjusted below as necessary
         self.compression = None
         self.readonly = False
@@ -358,6 +364,7 @@ class _File(object):
         self._mmap = None
 
         self.closed = True
+        self.close_on_error = False
 
     def _maybe_close_mmap(self, refcount_delta=0):
         """
@@ -497,6 +504,7 @@ class _File(object):
             self._file = bz2.BZ2File(self.name, bzip2_mode)
         else:
             self._file = fileobj_open(self.name, IO_FITS_MODES[mode])
+            self.close_on_error = True
 
         # Make certain we're back at the beginning of the file
         # BZ2File does not support seek when the file is open for writing, but
