@@ -14,7 +14,8 @@ from .. import conf
 from ..file import _File
 from ..header import Header, _pad_length
 from ..util import (_is_int, _is_pseudo_unsigned, _unsigned_zero,
-                    itersubclasses, decode_ascii, _get_array_mmap, first)
+                    itersubclasses, decode_ascii, _get_array_mmap, first,
+                    _free_space_check, _extract_number)
 from ..verify import _Verify, _ErrList
 
 from ....extern.six import string_types, add_metaclass
@@ -28,6 +29,8 @@ from ....utils.decorators import deprecated_renamed_argument
 
 class _Delayed(object):
     pass
+
+
 DELAYED = _Delayed()
 
 
@@ -86,6 +89,7 @@ def _hdu_class_from_header(cls, header):
 
     return klass
 
+
 class _BaseHDUMeta(type):
     def __init__(cls, name, bases, members):
         # The sole purpose of this metaclass right now is to add the same
@@ -323,7 +327,6 @@ class _BaseHDU(object):
         if not isinstance(fileobj, _File):
             fileobj = _File(fileobj)
 
-
         hdu = cls._readfrom_internal(fileobj, checksum=checksum,
                                      ignore_missing_end=ignore_missing_end,
                                      **kwargs)
@@ -333,7 +336,7 @@ class _BaseHDU(object):
         fileobj.seek(hdu._data_offset + hdu._data_size, os.SEEK_SET)
         return hdu
 
-    @deprecated_renamed_argument('clobber', 'overwrite', '1.3')
+    @deprecated_renamed_argument('clobber', 'overwrite', '2.0')
     def writeto(self, name, output_verify='exception', overwrite=False,
                 checksum=False):
         """
@@ -461,7 +464,6 @@ class _BaseHDU(object):
 
         return hdu
 
-
     def _get_raw_data(self, shape, code, offset):
         """
         Return raw array from either the HDU's memory buffer or underlying
@@ -489,7 +491,6 @@ class _BaseHDU(object):
         # Handle checksum
         self._update_checksum(checksum)
 
-
     def _update_uint_scale_keywords(self):
         """
         If the data is unsigned int 16, 32, or 64 add BSCALE/BZERO cards to
@@ -607,7 +608,6 @@ class _BaseHDU(object):
             # existing file or buffer
             size += self._writedata_direct_copy(fileobj)
 
-
         # flush, to make sure the content is written
         if not fileobj.simulateonly:
             fileobj.flush()
@@ -638,7 +638,6 @@ class _BaseHDU(object):
 
         If this proves too slow a more direct approach may be used.
         """
-
         raw = self._get_raw_data(self._data_size, 'ubyte', self._data_offset)
         if raw is not None:
             fileobj.writearray(raw)
@@ -651,6 +650,15 @@ class _BaseHDU(object):
     # HDUList, eventually the plan is to have this be moved into writeto()
     # somehow...
     def _writeto(self, fileobj, inplace=False, copy=False):
+        try:
+            dirname = os.path.dirname(fileobj._file.name)
+        except AttributeError:
+            dirname = None
+
+        with _free_space_check(self, dirname):
+            self._writeto_internal(fileobj, inplace, copy)
+
+    def _writeto_internal(self, fileobj, inplace, copy):
         # For now fileobj is assumed to be a _File object
         if not inplace or self._new:
             header_offset, _ = self._writeheader(fileobj)
@@ -692,12 +700,12 @@ class _BaseHDU(object):
         if self._data_loaded:
             if self.data is not None:
                 # Seek through the array's bases for an memmap'd array; we
-                # can't rely on the _File object to give us this info since the
-                # user may have replaced the previous mmap'd array
+                # can't rely on the _File object to give us this info since
+                # the user may have replaced the previous mmap'd array
                 if copy or self._data_replaced:
-                    # Of course, if we're copying the data to a new file we
-                    # don't care about flushing the original mmap; instead just
-                    # read it into the new file
+                    # Of course, if we're copying the data to a new file
+                    # we don't care about flushing the original mmap;
+                    # instead just read it into the new file
                     array_mmap = None
                 else:
                     array_mmap = _get_array_mmap(self.data)
@@ -724,6 +732,7 @@ class _BaseHDU(object):
                 _get_array_mmap(self.data) is not None):
             del self.data
 
+
 # For backwards-compatibility, though nobody should have
 # been using this directly:
 _AllHDU = _BaseHDU
@@ -766,7 +775,7 @@ class _CorruptedHDU(_BaseHDU):
         return self._file.size - self._data_offset
 
     def _summary(self):
-        return (self.name, 'CorruptedHDU')
+        return (self.name, self.ver, 'CorruptedHDU')
 
     def verify(self):
         pass
@@ -848,7 +857,7 @@ class _NonstandardHDU(_BaseHDU, _Verify):
         return offset, size
 
     def _summary(self):
-        return (self.name, 'NonstandardHDU', len(self._header))
+        return (self.name, self.ver, 'NonstandardHDU', len(self._header))
 
     @lazyproperty
     def data(self):
@@ -991,9 +1000,11 @@ class _ValidHDU(_BaseHDU, _Verify):
         naxis = self._header.get('NAXIS', 0)
         if naxis < 1000:
             for ax in range(3, naxis + 3):
-                self.req_cards('NAXIS' + str(ax - 2), ax,
-                               lambda v: (_is_int(v) and v >= 0), 1, option,
-                               errs)
+                key = 'NAXIS' + str(ax - 2)
+                self.req_cards(key, ax,
+                               lambda v: (_is_int(v) and v >= 0),
+                               _extract_number(self._header[key], default=1),
+                               option, errs)
 
             # Remove NAXISj cards where j is not in range 1, naxis inclusive.
             for keyword in self._header:
@@ -1572,7 +1583,7 @@ class ExtensionHDU(_ValidHDU):
 
         raise NotImplementedError
 
-    @deprecated_renamed_argument('clobber', 'overwrite', '1.3')
+    @deprecated_renamed_argument('clobber', 'overwrite', '2.0')
     def writeto(self, name, output_verify='exception', overwrite=False,
                 checksum=False):
         """
@@ -1603,6 +1614,8 @@ class ExtensionHDU(_ValidHDU):
                        1, option, errs)
 
         return errs
+
+
 # For backwards compatibility, though this needs to be deprecated
 # TODO: Mark this as deprecated
 _ExtensionHDU = ExtensionHDU
@@ -1643,7 +1656,7 @@ class NonstandardExtHDU(ExtensionHDU):
                 xtension not in standard_xtensions)
 
     def _summary(self):
-        return (self.name, 'NonstandardExtHDU', len(self._header))
+        return (self.name, self.ver, 'NonstandardExtHDU', len(self._header))
 
     @lazyproperty
     def data(self):
@@ -1653,5 +1666,6 @@ class NonstandardExtHDU(ExtensionHDU):
 
         return self._get_raw_data(self.size, 'ubyte', self._data_offset)
 
+
 # TODO: Mark this as deprecated
 _NonstandardExtHDU = NonstandardExtHDU
