@@ -7,7 +7,6 @@ import gzip
 import os
 import shutil
 import sys
-import textwrap
 import warnings
 
 import numpy as np
@@ -19,11 +18,11 @@ from .image import PrimaryHDU, ImageHDU
 from ..file import _File
 from ..header import _pad_length
 from ..util import (_is_int, _tmp_name, fileobj_closed, ignore_sigint,
-                    _get_array_mmap)
+                    _get_array_mmap, _free_space_check)
 from ..verify import _Verify, _ErrList, VerifyError, VerifyWarning
-from ....extern.six import string_types
+from ....extern.six import string_types, PY2
 from ....utils import indent
-from ....utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
+from ....utils.exceptions import AstropyUserWarning
 from ....utils.decorators import deprecated_renamed_argument
 from ....extern.six.moves import range
 
@@ -149,18 +148,11 @@ def fitsopen(name, mode='readonly', memmap=None, save_backup=False,
     else:
         lazy_load_hdus = bool(lazy_load_hdus)
 
-    if 'uint16' in kwargs and 'uint' not in kwargs:
-        kwargs['uint'] = kwargs['uint16']
-        del kwargs['uint16']
-        warnings.warn(
-            'The uint16 keyword argument is deprecated since v1.1.0.  Use '
-            'the uint argument instead.', AstropyDeprecationWarning)
-
     if 'uint' not in kwargs:
         kwargs['uint'] = conf.enable_uint
 
     if not name:
-        raise ValueError('Empty filename: {}'.format(repr(name)))
+        raise ValueError('Empty filename: {!r}'.format(name))
 
     return HDUList.fromfile(name, mode, memmap, save_backup, cache,
                             lazy_load_hdus, **kwargs)
@@ -216,10 +208,6 @@ class HDUList(list, _Verify):
         else:
             self._read_all = False
 
-        # This is used for book-keeping for backwards compatibility of
-        # accessing unread HDUs after the file has been closed.
-        self._total_read = None
-
         if hdus is None:
             hdus = []
 
@@ -304,14 +292,33 @@ class HDUList(list, _Verify):
                     if not self._read_next_hdu():
                         break
 
-            hdus = super(HDUList, self).__getitem__(key)
-            return HDUList(hdus)
+            try:
+                hdus = super(HDUList, self).__getitem__(key)
+            except IndexError as e:
+                # Raise a more helpful IndexError if the file was not fully read.
+                if self._read_all:
+                    raise e
+                else:
+                    raise IndexError('HDU not found, possibly because the index '
+                                     'is out of range, or because the file was '
+                                     'closed before all HDUs were read')
+            else:
+                return HDUList(hdus)
 
         # Originally this used recursion, but hypothetically an HDU with
         # a very large number of HDUs could blow the stack, so use a loop
         # instead
-        return self._try_while_unread_hdus(super(HDUList, self).__getitem__,
-                                           self._positive_index_of(key))
+        try:
+            return self._try_while_unread_hdus(super(HDUList, self).__getitem__,
+                                            self._positive_index_of(key))
+        except IndexError as e:
+            # Raise a more helpful IndexError if the file was not fully read.
+            if self._read_all:
+                raise e
+            else:
+                raise IndexError('HDU not found, possibly because the index '
+                                 'is out of range, or because the file was '
+                                 'closed before all HDUs were read')
 
     def __contains__(self, item):
         """
@@ -370,15 +377,15 @@ class HDUList(list, _Verify):
             self._truncate = False
             self._resize = True
 
-    def __getslice__(self, start, end):
-        return self[slice(start, end)]
+    if PY2:  # don't fall through to list.__getslice__, __delslice__
+        def __getslice__(self, start, end):
+            return self.__getitem__(slice(start, end))
 
-    def __delslice__(self, start, stop):
-        """
-        Delete a slice of HDUs from the `HDUList`, indexed by number only.
-        """
-
-        del self[slice(start, stop)]
+        def __delslice__(self, start, stop):
+            """
+            Delete a slice of HDUs from the `HDUList`, indexed by number only.
+            """
+            self.__delitem__(slice(start, stop))
 
     # Support the 'with' statement
     def __enter__(self):
@@ -680,7 +687,7 @@ class HDUList(list, _Verify):
                 break
 
         if (found is None):
-            raise KeyError('Extension {} not found.'.format(repr(key)))
+            raise KeyError('Extension {!r} not found.'.format(key))
         else:
             return found
 
@@ -774,13 +781,13 @@ class HDUList(list, _Verify):
                 # only append HDU's which are "new"
                 if hdu._new:
                     hdu._prewriteto(checksum=hdu._output_checksum)
-                    try:
+                    with _free_space_check(self):
                         hdu._writeto(self._file)
                         if verbose:
                             print('append HDU', hdu.name, extver)
                         hdu._new = False
-                    finally:
-                        hdu._postwriteto()
+                    hdu._postwriteto()
+
         elif self._file.mode == 'update':
             self._flush_update()
 
@@ -816,7 +823,7 @@ class HDUList(list, _Verify):
                 n = hdr['NAXIS']
                 hdr.set('EXTEND', True, after='NAXIS' + str(n))
 
-    @deprecated_renamed_argument('clobber', 'overwrite', '1.3')
+    @deprecated_renamed_argument('clobber', 'overwrite', '2.0')
     def writeto(self, fileobj, output_verify='exception', overwrite=False,
                 checksum=False):
         """
@@ -868,14 +875,16 @@ class HDUList(list, _Verify):
         # but only if the file doesn't exist.
         fileobj = _File(fileobj, mode='ostream', overwrite=overwrite)
         hdulist = self.fromfile(fileobj)
+        try:
+            dirname = os.path.dirname(hdulist._file.name)
+        except AttributeError:
+            dirname = None
 
-        for hdu in self:
-            hdu._prewriteto(checksum=checksum)
-            try:
+        with _free_space_check(self, dirname=dirname):
+            for hdu in self:
+                hdu._prewriteto(checksum=checksum)
                 hdu._writeto(hdulist._file)
-            finally:
                 hdu._postwriteto()
-
         hdulist.close(output_verify=output_verify, closed=closed)
 
     def close(self, output_verify='exception', verbose=False, closed=True):
@@ -898,45 +907,17 @@ class HDUList(list, _Verify):
             When `True`, close the underlying file object.
         """
 
-        # If the underlying file object is not closed then we can still
-        # technically lazy-load HDUs
-
-        if closed:
-            # Get the actual current number of HDUs that have been loaded
-            total_read = list.__len__(self)
-
-            # Now read in any remaining HDUs
-            while self._read_next_hdu():
-                pass
-
         try:
-            self._close(output_verify=output_verify, verbose=verbose,
-                        closed=closed)
-        finally:
-            # This will cause deprecation warnings on any future attempt
-            # to access HDUs that were not loaded before closing
-            if closed:
-                self._total_read = total_read
-
-    def _close(self, output_verify='exception', verbose=False, closed=True):
-        """
-        Internal implementation of close() that does not allow further
-        HDUs to be loaded.
-
-        The HDU pre-loading behavior of the explicit close() is currently
-        maintained only for backwards-compatibility.
-        """
-
-        if self._file:
-            if self._file.mode in ['append', 'update']:
+            if (self._file and self._file.mode in ('append', 'update')
+                    and not self._file.closed):
                 self.flush(output_verify=output_verify, verbose=verbose)
-
-            if closed and hasattr(self._file, 'close'):
+        finally:
+            if self._file and closed and hasattr(self._file, 'close'):
                 self._file.close()
 
-        # Give individual HDUs an opportunity to do on-close cleanup
-        for hdu in self:
-            hdu._close(closed=closed)
+            # Give individual HDUs an opportunity to do on-close cleanup
+            for hdu in self:
+                hdu._close(closed=closed)
 
     def info(self, output=None):
         """
@@ -962,10 +943,10 @@ class HDUList(list, _Verify):
             name = self._file.name
 
         results = ['Filename: {}'.format(name),
-                   'No.    Name         Type      Cards   Dimensions   Format']
+                   'No.    Name      Ver    Type      Cards   Dimensions   Format']
 
-        format = '{:3d}  {:10}  {:11}  {:5d}   {}   {}   {}'
-        default = ('', '', 0, (), '', '')
+        format = '{:3d}  {:10}  {:3} {:11}  {:5d}   {}   {}   {}'
+        default = ('', '', '', 0, (), '', '')
         for idx, hdu in enumerate(self):
             summary = hdu._summary()
             if len(summary) < len(default):
@@ -1038,9 +1019,14 @@ class HDUList(list, _Verify):
 
         # Make sure at least the PRIMARY HDU can be read
         read_one = hdulist._read_next_hdu()
+
         # If we're trying to read only and no header units were found,
-        # raise and exception
+        # raise an exception
         if not read_one and mode in ('readonly', 'denywrite'):
+            # Close the file if necessary (issue #6168)
+            if hdulist._file.close_on_error:
+                hdulist._file.close()
+
             raise IOError('Empty or corrupt FITS file')
 
         if not lazy_load_hdus:
@@ -1054,45 +1040,17 @@ class HDUList(list, _Verify):
 
         return hdulist
 
-    def _try_while_unread_hdus(self, func, index, *args, **kwargs):
+    def _try_while_unread_hdus(self, func, *args, **kwargs):
         """
         Attempt an operation that accesses an HDU by index/name
         that can fail if not all HDUs have been read yet.  Keep
         reading HDUs until the operation succeeds or there are no
         more HDUs to read.
-
-        The first argument must always be the index of some HDU in the
-        HDUList, but additional arguments may be passed on.
         """
 
-        # TODO: This is a temporary hack for reporting the deprecation
-        # warning on the pre-lazy-loading behavior of allowing HDUs to
-        # be accessed after the file was closed.  It can be removed once
-        # the deprecation period has passed.
-        if self._file and self._file.closed and self._total_read is not None:
-            if index > self._total_read - 1:
-                warnings.warn(textwrap.dedent(
-                    """\
-                    Accessing an HDU after an HDUList is closed, where
-                    that HDU was no read while the HDUList was open
-                    is deprecated.  That is, you did something like:
-
-                        >>> hdulist.close()
-                        >>> print(hdulist[2].header)
-
-                    even though hdulist[2] had not been read yet.  Instead
-                    do:
-
-                        >>> print(hdulist[2].header)
-                        >>> hdulist.close()
-
-                    or open the file with lazy_load_hdus=False to read all
-                    the HDUs into memory immediately upon opening the file.
-                    """), AstropyDeprecationWarning)
-
         while True:
             try:
-                return func(index, *args, **kwargs)
+                return func(*args, **kwargs)
             except Exception:
                 if self._read_next_hdu():
                     continue
@@ -1113,6 +1071,9 @@ class HDUList(list, _Verify):
         saved_compression_enabled = compressed.COMPRESSION_ENABLED
         fileobj, data, kwargs = self._file, self._data, self._open_kwargs
 
+        if fileobj is not None and fileobj.closed:
+            return False
+
         try:
             self._in_read_next_hdu = True
 
@@ -1136,15 +1097,13 @@ class HDUList(list, _Verify):
                     except EOFError:
                         self._read_all = True
                         return False
-                    except ValueError:
-                        # A ValueError can occur when trying to perform I/O
-                        # on a closed file
-                        if fileobj.closed:
-                            self._read_all = True
-                            return False
-                        else:
-                            raise
                     except IOError:
+                        # Close the file: see
+                        # https://github.com/astropy/astropy/issues/6168
+                        #
+                        if self._file.close_on_error:
+                            self._file.close()
+
                         if fileobj.writeonly:
                             self._read_all = True
                             return False
@@ -1185,7 +1144,6 @@ class HDUList(list, _Verify):
         return True
 
     def _verify(self, option='warn'):
-        text = ''
         errs = _ErrList([], unit='HDU')
 
         # the first (0th) element must be a primary HDU
@@ -1282,20 +1240,18 @@ class HDUList(list, _Verify):
             else:
                 new_file = name
 
-            hdulist = self.fromfile(new_file, mode='append')
+            with self.fromfile(new_file, mode='append') as hdulist:
 
-            for hdu in self:
-                hdu._writeto(hdulist._file, inplace=True, copy=True)
-
-            if sys.platform.startswith('win'):
-                # Collect a list of open mmaps to the data; this well be used
-                # later.  See below.
-                mmaps = [(idx, _get_array_mmap(hdu.data), hdu.data)
-                         for idx, hdu in enumerate(self) if hdu._has_data]
-
-            hdulist._file.close()
-            self._file.close()
+                for hdu in self:
+                    hdu._writeto(hdulist._file, inplace=True, copy=True)
+                if sys.platform.startswith('win'):
+                    # Collect a list of open mmaps to the data; this well be
+                    # used later.  See below.
+                    mmaps = [(idx, _get_array_mmap(hdu.data), hdu.data)
+                             for idx, hdu in enumerate(self) if hdu._has_data]
 
+                hdulist._file.close()
+                self._file.close()
             if sys.platform.startswith('win'):
                 # Close all open mmaps to the data.  This is only necessary on
                 # Windows, which will not allow a file to be renamed or deleted
