@@ -12,7 +12,7 @@ from ..verify import VerifyWarning
 
 from ....extern.six import string_types
 from ....extern.six.moves import range, zip
-from ....utils import isiterable, lazyproperty, classproperty, deprecated
+from ....utils import isiterable, lazyproperty
 
 
 class _ImageBaseHDU(_ValidHDU):
@@ -73,17 +73,17 @@ class _ImageBaseHDU(_ValidHDU):
                 c0 = ('SIMPLE', True, self.standard_keyword_comments['SIMPLE'])
             cards = [
                 c0,
-                ('BITPIX',    8, self.standard_keyword_comments['BITPIX']),
-                ('NAXIS',     0, self.standard_keyword_comments['NAXIS'])]
+                ('BITPIX', 8, self.standard_keyword_comments['BITPIX']),
+                ('NAXIS', 0, self.standard_keyword_comments['NAXIS'])]
 
             if isinstance(self, GroupsHDU):
                 cards.append(('GROUPS', True,
                              self.standard_keyword_comments['GROUPS']))
 
             if isinstance(self, (ExtensionHDU, GroupsHDU)):
-                cards.append(('PCOUNT',    0,
+                cards.append(('PCOUNT', 0,
                               self.standard_keyword_comments['PCOUNT']))
-                cards.append(('GCOUNT',    1,
+                cards.append(('GCOUNT', 1,
                               self.standard_keyword_comments['GCOUNT']))
 
             if header is not None:
@@ -170,7 +170,7 @@ class _ImageBaseHDU(_ValidHDU):
         # but there should be a BSCALE/BZERO now that the data has been set.
         if not bzero_in_header:
             self._orig_bzero = self._bzero
-        if bscale_in_header:
+        if not bscale_in_header:
             self._orig_bscale = self._bscale
 
     @classmethod
@@ -303,7 +303,6 @@ class _ImageBaseHDU(_ValidHDU):
         self._orig_bscale = self._bscale
         self._orig_bzero = self._bzero
 
-
         # returning the data signals to lazyproperty that we've already handled
         # setting self.__dict__['data']
         return data
@@ -428,7 +427,7 @@ class _ImageBaseHDU(_ValidHDU):
         self._bitpix = self._header['BITPIX']
         self._blank = self._header.pop('BLANK', None)
 
-    def scale(self, type=None, option='old', bscale=1, bzero=0):
+    def scale(self, type=None, option='old', bscale=None, bzero=None):
         """
         Scale image data by using ``BSCALE``/``BZERO``.
 
@@ -444,12 +443,12 @@ class _ImageBaseHDU(_ValidHDU):
             dtype name, (e.g. ``'uint8'``, ``'int16'``, ``'float32'``
             etc.).  If is `None`, use the current data type.
 
-        option : str
-            How to scale the data: if ``"old"``, use the original
-            ``BSCALE`` and ``BZERO`` values when the data was
-            read/created. If ``"minmax"``, use the minimum and maximum
-            of the data to scale.  The option will be overwritten by
-            any user specified ``bscale``/``bzero`` values.
+        option : str, optional
+            How to scale the data: ``"old"`` uses the original ``BSCALE`` and
+            ``BZERO`` values from when the data was read/created (defaulting to
+            1 and 0 if they don't exist). For integer data only, ``"minmax"``
+            uses the minimum and maximum of the data to scale. User-specified
+            ``bscale``/``bzero`` values always take precedence.
 
         bscale, bzero : int, optional
             User-specified ``BSCALE`` and ``BZERO`` values
@@ -459,7 +458,7 @@ class _ImageBaseHDU(_ValidHDU):
         self._scale_internal(type=type, option=option, bscale=bscale,
                              bzero=bzero, blank=None)
 
-    def _scale_internal(self, type=None, option='old', bscale=1, bzero=0,
+    def _scale_internal(self, type=None, option='old', bscale=None, bzero=None,
                         blank=0):
         """
         This is an internal implementation of the `scale` method, which
@@ -486,31 +485,35 @@ class _ImageBaseHDU(_ValidHDU):
 
         # Determine how to scale the data
         # bscale and bzero takes priority
-        if (bscale != 1 or bzero != 0):
+        if bscale is not None and bzero is not None:
             _scale = bscale
             _zero = bzero
+        elif bscale is not None:
+            _scale = bscale
+            _zero = 0
+        elif bzero is not None:
+            _scale = 1
+            _zero = bzero
+        elif (option == 'old' and self._orig_bscale is not None and
+                self._orig_bzero is not None):
+            _scale = self._orig_bscale
+            _zero = self._orig_bzero
+        elif option == 'minmax' and not issubclass(_type, np.floating):
+            min = np.minimum.reduce(self.data.flat)
+            max = np.maximum.reduce(self.data.flat)
+
+            if _type == np.uint8:  # uint8 case
+                _zero = min
+                _scale = (max - min) / (2.0 ** 8 - 1)
+            else:
+                _zero = (max + min) / 2.0
+
+                # throw away -2^N
+                nbytes = 8 * _type().itemsize
+                _scale = (max - min) / (2.0 ** nbytes - 2)
         else:
-            if option == 'old':
-                _scale = self._orig_bscale
-                _zero = self._orig_bzero
-            elif option == 'minmax':
-                if issubclass(_type, np.floating):
-                    _scale = 1
-                    _zero = 0
-                else:
-
-                    min = np.minimum.reduce(self.data.flat)
-                    max = np.maximum.reduce(self.data.flat)
-
-                    if _type == np.uint8:  # uint8 case
-                        _zero = min
-                        _scale = (max - min) / (2.0 ** 8 - 1)
-                    else:
-                        _zero = (max + min) / 2.0
-
-                        # throw away -2^N
-                        nbytes = 8 * _type().itemsize
-                        _scale = (max - min) / (2.0 ** nbytes - 2)
+            _scale = 1
+            _zero = 0
 
         # Do the scaling
         if _zero != 0:
@@ -630,12 +633,18 @@ class _ImageBaseHDU(_ValidHDU):
                 should_swap = (byteorder in swap_types)
 
             if not fileobj.simulateonly:
+
                 if should_swap:
-                    output.byteswap(True)
-                    try:
-                        fileobj.writearray(output)
-                    finally:
+                    if output.flags.writeable:
                         output.byteswap(True)
+                        try:
+                            fileobj.writearray(output)
+                        finally:
+                            output.byteswap(True)
+                    else:
+                        # For read-only arrays, there is no way around making
+                        # a byteswapped copy of the data.
+                        fileobj.writearray(output.byteswap(False))
                 else:
                     fileobj.writearray(output)
 
@@ -792,7 +801,7 @@ class _ImageBaseHDU(_ValidHDU):
         # Display shape in FITS-order
         shape = tuple(reversed(self.shape))
 
-        return (self.name, class_name, len(self._header), shape, format, '')
+        return (self.name, self.ver, class_name, len(self._header), shape, format, '')
 
     def _calculate_datasum(self, blocking):
         """
@@ -800,6 +809,7 @@ class _ImageBaseHDU(_ValidHDU):
         """
 
         if self._has_data:
+
             # We have the data to be used.
             d = self.data
 
@@ -812,9 +822,16 @@ class _ImageBaseHDU(_ValidHDU):
             # Check the byte order of the data.  If it is little endian we
             # must swap it before calculating the datasum.
             if d.dtype.str[0] != '>':
-                byteswapped = True
-                d = d.byteswap(True)
-                d.dtype = d.dtype.newbyteorder('>')
+                if d.flags.writeable:
+                    byteswapped = True
+                    d = d.byteswap(True)
+                    d.dtype = d.dtype.newbyteorder('>')
+                else:
+                    # If the data is not writeable, we just make a byteswapped
+                    # copy and don't bother changing it back after
+                    d = d.byteswap(False)
+                    d.dtype = d.dtype.newbyteorder('>')
+                    byteswapped = False
             else:
                 byteswapped = False
 
@@ -836,18 +853,6 @@ class _ImageBaseHDU(_ValidHDU):
             return super(_ImageBaseHDU, self)._calculate_datasum(
                 blocking=blocking)
 
-    @classproperty
-    @deprecated('1.1.0', alternative='the module level constant BITPIX2DTYPE',
-                obj_type='class attribute')
-    def NumCode(cls):
-        return BITPIX2DTYPE
-
-    @classproperty
-    @deprecated('1.1.0', alternative='the module level constant DTYPE2BITPIX',
-                obj_type='class attribute')
-    def ImgCode(cls):
-        return DTYPE2BITPIX
-
 
 class Section(object):
     """
@@ -1007,8 +1012,10 @@ class PrimaryHDU(_ImageBaseHDU):
     @classmethod
     def match_header(cls, header):
         card = header.cards[0]
+        # Due to problems discussed in #5808, we cannot assume the 'GROUPS'
+        # keyword to be True/False, have to check the value
         return (card.keyword == 'SIMPLE' and
-                ('GROUPS' not in header or not header['GROUPS']) and
+                ('GROUPS' not in header or header['GROUPS'] != True) and  # noqa
                 card.value)
 
     def update_header(self):
