@@ -17,6 +17,8 @@ import textwrap
 import threading
 import warnings
 import weakref
+from contextlib import contextmanager
+from ...utils import data
 
 from distutils.version import LooseVersion
 
@@ -590,6 +592,7 @@ def fill(text, width, *args, **kwargs):
 # time it is needed.
 CHUNKED_FROMFILE = None
 
+
 def _array_from_file(infile, dtype, count, sep):
     """Create a numpy array from a file or a file-like object."""
 
@@ -627,6 +630,7 @@ def _array_from_file(infile, dtype, count, sep):
 _OSX_WRITE_LIMIT = (2 ** 32) - 1
 _WIN_WRITE_LIMIT = (2 ** 31) - 1
 
+
 def _array_to_file(arr, outfile):
     """
     Write a numpy array to a file or a file-like object.
@@ -644,7 +648,6 @@ def _array_to_file(arr, outfile):
     `ndarray.tofile`.  Otherwise a slower Python implementation is used.
     """
 
-
     if isfile(outfile):
         write = lambda a, f: a.tofile(f)
     else:
@@ -857,3 +860,40 @@ def _get_array_mmap(array):
         if isinstance(base.base, mmap.mmap):
             return base.base
         base = base.base
+
+
+@contextmanager
+def _free_space_check(hdulist, dirname=None):
+    try:
+        yield
+    except OSError as exc:
+        error_message = ''
+        if not isinstance(hdulist, list):
+            hdulist = [hdulist, ]
+        if dirname is None:
+            dirname = os.path.dirname(hdulist._file.name)
+        if os.path.isdir(dirname):
+            free_space = data.get_free_space_in_dir(dirname)
+            hdulist_size = np.sum(hdu.size for hdu in hdulist)
+            if free_space < hdulist_size:
+                error_message = ("Not enough space on disk: requested {}, "
+                                 "available {}. ".format(hdulist_size, free_space))
+
+        for hdu in hdulist:
+            hdu._close()
+
+        raise OSError(error_message + str(exc))
+
+
+def _extract_number(value, default):
+    """
+    Attempts to extract an integer number from the given value. If the
+    extraction fails, the value of the 'default' argument is returned.
+    """
+
+    try:
+        # The _str_to_num method converts the value to string/float
+        # so we need to perform one additional conversion to int on top
+        return int(_str_to_num(value))
+    except (TypeError, ValueError):
+        return default
