@@ -15,6 +15,7 @@ from .polynomial import *
 from .functional_models import *
 from .powerlaws import *
 from .tabular import *
+from .blackbody import BlackBody1D
 
 from ..extern.six import string_types
 
@@ -57,7 +58,7 @@ CONSTRAINTS_DOC = """
 MODELS_WITH_CONSTRAINTS = [
     AiryDisk2D, Moffat1D, Moffat2D, Box1D, Box2D,
     Const1D, Const2D, Ellipse2D, Disk2D,
-    Gaussian1D, GaussianAbsorption1D, Gaussian2D,
+    Gaussian1D, Gaussian2D,
     Linear1D, Lorentz1D, MexicanHat1D, MexicanHat2D,
     PowerLaw1D, Sersic1D, Sersic2D, Sine1D, Trapezoid1D, TrapezoidDisk2D,
     Chebyshev1D, Chebyshev2D, Hermite1D, Hermite2D, Legendre2D, Legendre1D,
