@@ -1,28 +1,32 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-This module contains simple statistical algorithms that are straightforwardly
-implemented as a single python function (or family of functions).
+This module contains simple statistical algorithms that are
+straightforwardly implemented as a single python function (or family of
+functions).
 
-This module should generally not be used directly.  Everything in `__all__` is
-imported into `astropy.stats`, and hence that package should be used for
-access.
+This module should generally not be used directly.  Everything in
+`__all__` is imported into `astropy.stats`, and hence that package
+should be used for access.
 """
 
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
 import math
-
 import numpy as np
 
+from warnings import warn
+
+from ..utils.compat import NUMPY_LT_1_10
+from ..utils.decorators import deprecated_renamed_argument
+from ..utils import isiterable
 from ..extern.six.moves import range
 
 
-__all__ = ['binom_conf_interval', 'binned_binom_proportion',
-           'poisson_conf_interval', 'median_absolute_deviation',
-           'mad_std', 'biweight_location', 'biweight_midvariance',
-           'signal_to_noise_oir_ccd', 'bootstrap', 'gaussian_fwhm_to_sigma',
-           'gaussian_sigma_to_fwhm']
+__all__ = ['gaussian_fwhm_to_sigma', 'gaussian_sigma_to_fwhm',
+           'binom_conf_interval', 'binned_binom_proportion',
+           'poisson_conf_interval', 'median_absolute_deviation', 'mad_std',
+           'signal_to_noise_oir_ccd', 'bootstrap']
 
 __doctest_skip__ = ['binned_binom_proportion']
 __doctest_requires__ = {'binom_conf_interval': ['scipy.special'],
@@ -33,14 +37,14 @@ __doctest_requires__ = {'binom_conf_interval': ['scipy.special'],
 
 gaussian_sigma_to_fwhm = 2.0 * np.sqrt(2.0 * np.log(2.0))
 """
-Factor with which to multiply Gaussian 1-sigma standard deviation(s) to
-convert them to full width at half maximum(s).
+Factor with which to multiply Gaussian 1-sigma standard deviation to
+convert it to full width at half maximum (FWHM).
 """
 
 gaussian_fwhm_to_sigma = 1. / gaussian_sigma_to_fwhm
 """
-Factor with which to multiply Gaussian full width at half maximum(s) to
-convert them to 1-sigma standard deviation(s).
+Factor with which to multiply Gaussian full width at half maximum (FWHM)
+to convert it to 1-sigma standard deviation.
 """
 
 
@@ -718,7 +722,8 @@ def poisson_conf_interval(n, interval='root-n', sigma=1, background=0,
     return conf_interval
 
 
-def median_absolute_deviation(a, axis=None):
+@deprecated_renamed_argument('a', 'data', '2.0')
+def median_absolute_deviation(data, axis=None, func=None, ignore_nan=False):
     """
     Calculate the median absolute deviation (MAD).
 
@@ -726,11 +731,19 @@ def median_absolute_deviation(a, axis=None):
 
     Parameters
     ----------
-    a : array-like
+    data : array-like
         Input array or object that can be converted to an array.
-    axis : int, optional
+    axis : {int, sequence of int, None}, optional
         Axis along which the MADs are computed.  The default (`None`) is
         to compute the MAD of the flattened array.
+    func : callable, optional
+        The function used to compute the median. Defaults to `numpy.ma.median`
+        for masked arrays, otherwise to `numpy.median`.
+    ignore_nan : bool
+        Ignore NaN values (treat them as if they are not in the array) when
+        computing the median.  This will use `numpy.ma.median` if ``axis`` is
+        specified, or `numpy.nanmedian` if ``axis==None`` and numpy's version
+        is >1.10 because nanmedian is slightly faster in this case.
 
     Returns
     -------
@@ -757,26 +770,57 @@ def median_absolute_deviation(a, axis=None):
     mad_std
     """
 
-    # Check if the array has a mask and if so use np.ma.median
-    # See https://github.com/numpy/numpy/issues/7330 why using np.ma.median
-    # for normal arrays should not be done (summary: np.ma.median always
-    # returns an masked array even if the result should be scalar). (#4658)
-    if isinstance(a, np.ma.MaskedArray):
-        func = np.ma.median
+    if func is None:
+        # Check if the array has a mask and if so use np.ma.median
+        # See https://github.com/numpy/numpy/issues/7330 why using np.ma.median
+        # for normal arrays should not be done (summary: np.ma.median always
+        # returns an masked array even if the result should be scalar). (#4658)
+        if isinstance(data, np.ma.MaskedArray):
+            is_masked = True
+            func = np.ma.median
+            if ignore_nan:
+                data = np.ma.masked_invalid(data)
+        elif ignore_nan:
+            is_masked = False
+            func = np.nanmedian
+        else:
+            is_masked = False
+            func = np.median
     else:
-        func = np.median
+        is_masked = None
 
-    a = np.asanyarray(a)
-    a_median = func(a, axis=axis)
+    if not ignore_nan and NUMPY_LT_1_10 and np.any(np.isnan(data)):
+        warn("Numpy versions <1.10 will return a number rather than NaN for "
+             "the median of arrays containing NaNs.  This behavior is "
+             "unlikely to be what you expect.")
+
+    data = np.asanyarray(data)
+    # np.nanmedian has `keepdims`, which is a good option if we're not allowing
+    # user-passed functions here
+    data_median = func(data, axis=axis)
 
     # broadcast the median array before subtraction
     if axis is not None:
-        a_median = np.expand_dims(a_median, axis=axis)
+        if isiterable(axis):
+            for ax in sorted(list(axis)):
+                data_median = np.expand_dims(data_median, axis=ax)
+        else:
+            data_median = np.expand_dims(data_median, axis=axis)
 
-    return func(np.abs(a - a_median), axis=axis)
+    result = func(np.abs(data - data_median), axis=axis, overwrite_input=True)
+
+    if axis is None and np.ma.isMaskedArray(result):
+        # return scalar version
+        result = result.item()
+    elif np.ma.isMaskedArray(result) and not is_masked:
+        # if the input array was not a masked array, we don't want to return a
+        # masked array
+        result = result.filled(fill_value=np.nan)
+
+    return result
 
 
-def mad_std(data, axis=None):
+def mad_std(data, axis=None, func=None, ignore_nan=False):
     r"""
     Calculate a robust standard deviation using the `median absolute
     deviation (MAD)
@@ -796,10 +840,18 @@ def mad_std(data, axis=None):
     ----------
     data : array-like
         Data array or object that can be converted to an array.
-    axis : int, optional
+    axis : {int, sequence of int, None}, optional
         Axis along which the robust standard deviations are computed.
         The default (`None`) is to compute the robust standard deviation
         of the flattened array.
+    func : callable, optional
+        The function used to compute the median. Defaults to `numpy.ma.median`
+        for masked arrays, otherwise to `numpy.median`.
+    ignore_nan : bool
+        Ignore NaN values (treat them as if they are not in the array) when
+        computing the median.  This will use `numpy.ma.median` if ``axis`` is
+        specified, or `numpy.nanmedian` if ``axis=None`` and numpy's version is
+        >1.10 because nanmedian is slightly faster in this case.
 
     Returns
     -------
@@ -819,205 +871,12 @@ def mad_std(data, axis=None):
 
     See Also
     --------
-    biweight_midvariance, median_absolute_deviation
+    biweight_midvariance, biweight_midcovariance, median_absolute_deviation
     """
 
     # NOTE: 1. / scipy.stats.norm.ppf(0.75) = 1.482602218505602
-    return median_absolute_deviation(data, axis=axis) * 1.482602218505602
-
-
-def biweight_location(a, c=6.0, M=None, axis=None):
-    r"""
-    Compute the biweight location.
-
-    The biweight location is a robust statistic for determining the
-    central location of a distribution.  It is given by:
-
-    .. math::
-
-        C_{bl}= M+\frac{\Sigma_{\|u_i\|<1} (x_i-M)(1-u_i^2)^2}
-        {\Sigma_{\|u_i\|<1} (1-u_i^2)^2}
-
-    where :math:`M` is the sample median (or the input initial guess)
-    and :math:`u_i` is given by:
-
-    .. math::
-
-        u_{i} = \frac{(x_i-M)}{c\ MAD}
-
-    where :math:`c` is the tuning constant and :math:`MAD` is the median
-    absolute deviation.
-
-    For more details, see `Beers, Flynn, and Gebhardt (1990); AJ 100, 32
-    <http://adsabs.harvard.edu/abs/1990AJ....100...32B>`_.
-
-    Parameters
-    ----------
-    a : array-like
-        Input array or object that can be converted to an array.
-    c : float, optional
-        Tuning constant for the biweight estimator.  Default value is 6.0.
-    M : float or array-like, optional
-        Initial guess for the biweight location.  An array can be input
-        when using the ``axis`` keyword.
-    axis : int, optional
-        Axis along which the biweight locations are computed.  The
-        default (`None`) is to compute the biweight location of the
-        flattened array.
-
-    Returns
-    -------
-    biweight_location : float or `~numpy.ndarray`
-        The biweight location of the input data.  If ``axis`` is `None`
-        then a scalar will be returned, otherwise a `~numpy.ndarray`
-        will be returned.
-
-    Examples
-    --------
-    Generate random variates from a Gaussian distribution and return the
-    biweight location of the distribution::
-
-        >>> import numpy as np
-        >>> from astropy.stats import biweight_location
-        >>> rand = np.random.RandomState(12345)
-        >>> from numpy.random import randn
-        >>> loc = biweight_location(rand.randn(1000))
-        >>> print(loc)    # doctest: +FLOAT_CMP
-        -0.0175741540445
-
-    See Also
-    --------
-    biweight_midvariance, median_absolute_deviation, mad_std
-    """
-
-    a = np.asanyarray(a)
-
-    if M is None:
-        M = np.median(a, axis=axis)
-    if axis is not None:
-        M = np.expand_dims(M, axis=axis)
-
-    # set up the differences
-    d = a - M
-
-    # set up the weighting
-    mad = median_absolute_deviation(a, axis=axis)
-    if axis is not None:
-        mad = np.expand_dims(mad, axis=axis)
-    u = d / (c * mad)
-
-    # now remove the outlier points
-    mask = (np.abs(u) >= 1)
-    u = (1 - u ** 2) ** 2
-    u[mask] = 0
-
-    return M.squeeze() + (d * u).sum(axis=axis) / u.sum(axis=axis)
-
-
-def biweight_midvariance(a, c=9.0, M=None, axis=None):
-    r"""
-    Compute the biweight midvariance.
-
-    The biweight midvariance is a robust statistic for determining the
-    midvariance (i.e. the standard deviation) of a distribution.  It is
-    given by:
-
-    .. math::
-
-      C_{bl}= (n')^{1/2} \frac{[\Sigma_{|u_i|<1} (x_i-M)^2(1-u_i^2)^4]^{0.5}}
-      {|\Sigma_{|u_i|<1} (1-u_i^2)(1-5u_i^2)|}
-
-    where :math:`u_i` is given by
-
-    .. math::
-
-        u_{i} = \frac{(x_i-M)}{c MAD}
-
-    where :math:`c` is the tuning constant and :math:`MAD` is the median
-    absolute deviation.  The midvariance tuning constant ``c`` is
-    typically 9.0.
-
-    :math:`n'` is the number of points for which :math:`|u_i| < 1`
-    holds, while the summations are over all :math:`i` up to :math:`n`:
-
-    .. math::
-
-        n' = \Sigma_{|u_i|<1}^n 1
-
-    This is slightly different than given in the reference below, but
-    results in a value closer to the true midvariance.
-
-    For more details, see `Beers, Flynn, and Gebhardt (1990); AJ 100, 32
-    <http://adsabs.harvard.edu/abs/1990AJ....100...32B>`_.
-
-    Parameters
-    ----------
-    a : array-like
-        Input array or object that can be converted to an array.
-    c : float, optional
-        Tuning constant for the biweight estimator.  Default value is 9.0.
-    M : float or array-like, optional
-        Initial guess for the biweight location.  An array can be input
-        when using the ``axis`` keyword.
-    axis : int, optional
-        Axis along which the biweight midvariances are computed.  The
-        default (`None`) is to compute the biweight midvariance of the
-        flattened array.
-
-    Returns
-    -------
-    biweight_midvariance : float or `~numpy.ndarray`
-        The biweight midvariance of the input data.  If ``axis`` is
-        `None` then a scalar will be returned, otherwise a
-        `~numpy.ndarray` will be returned.
-
-    Examples
-    --------
-    Generate random variates from a Gaussian distribution and return the
-    biweight midvariance of the distribution::
-
-        >>> import numpy as np
-        >>> from astropy.stats import biweight_midvariance
-        >>> rand = np.random.RandomState(12345)
-        >>> from numpy.random import randn
-        >>> bmv = biweight_midvariance(rand.randn(1000))
-        >>> print(bmv)    # doctest: +FLOAT_CMP
-        0.986726249291
-
-    See Also
-    --------
-    biweight_location, mad_std, median_absolute_deviation
-    """
-
-    a = np.asanyarray(a)
-
-    if M is None:
-        M = np.median(a, axis=axis)
-    if axis is not None:
-        M = np.expand_dims(M, axis=axis)
-
-    # set up the differences
-    d = a - M
-
-    # set up the weighting
-    mad = median_absolute_deviation(a, axis=axis)
-    if axis is not None:
-        mad = np.expand_dims(mad, axis=axis)
-    u = d / (c * mad)
-
-    # now remove the outlier points
-    mask = np.abs(u) < 1
-    u = u ** 2
-    n = mask.sum(axis=axis)
-
-    f1 = d * d * (1. - u)**4
-    f1[~mask] = 0.
-    f1 = f1.sum(axis=axis) ** 0.5
-    f2 = (1. - u) * (1. - 5.*u)
-    f2[~mask] = 0.
-    f2 = np.abs(f2.sum(axis=axis))
-
-    return (n ** 0.5) * f1 / f2
+    MAD = median_absolute_deviation(data, axis=axis, func=func, ignore_nan=ignore_nan)
+    return MAD * 1.482602218505602
 
 
 def signal_to_noise_oir_ccd(t, source_eps, sky_eps, dark_eps, rd, npix,
@@ -1150,8 +1009,8 @@ def bootstrap(data, bootnum=100, samples=None, bootfunc=None):
         samples = data.shape[0]
 
     # make sure the input is sane
-    assert samples > 0, "samples cannot be less than one"
-    assert bootnum > 0, "bootnum cannot be less than one"
+    if samples < 1 or bootnum < 1:
+        raise ValueError("neither 'samples' nor 'bootnum' can be less than 1.")
 
     if bootfunc is None:
         resultdims = (bootnum,) + (samples,) + data.shape[1:]
