@@ -12,6 +12,7 @@ from ..utils.data_info import DataInfo
 
 __all__ = ['table_info', 'TableInfo']
 
+
 def table_info(tbl, option='attributes', out=''):
     """
     Write summary information about column to the ``out`` filehandle.
@@ -53,9 +54,9 @@ def table_info(tbl, option='attributes', out=''):
     Parameters
     ----------
     option : str, function, list of (str or function)
-        Info option (default='attributes')
+        Info option, defaults to 'attributes'.
     out : file-like object, None
-        Output destination (default=sys.stdout).  If None then a
+        Output destination, default is sys.stdout.  If None then a
         Table with information attributes is returned
 
     Returns
@@ -114,6 +115,7 @@ def table_info(tbl, option='attributes', out=''):
 
     out.writelines(outline + os.linesep for outline in outlines)
 
+
 class TableInfo(DataInfo):
     _parent = None
 
