@@ -20,7 +20,7 @@ __all__ = []
 
 def default_format_func(format_, val):
     if isinstance(val, bytes):
-        return val.decode('utf-8')
+        return val.decode('utf-8', errors='replace')
     else:
         return text_type(val)
 
@@ -28,7 +28,7 @@ def default_format_func(format_, val):
 _format_funcs = {}
 
 
-### The first three functions are helpers for _auto_format_func
+# The first three functions are helpers for _auto_format_func
 
 
 def _use_str_for_masked_values(format_func):
@@ -40,6 +40,7 @@ def _use_str_for_masked_values(format_func):
     return lambda format_, val: (str(val) if val is np.ma.masked
                                  else format_func(format_, val))
 
+
 def _possible_string_format_functions(format_):
     """Iterate through possible string-derived format functions.
 
@@ -50,6 +51,7 @@ def _possible_string_format_functions(format_):
     yield lambda format_, val: format_.format(val)
     yield lambda format_, val: format_ % val
 
+
 def get_auto_format_func(
         col_name=None,
         possible_string_format_functions=_possible_string_format_functions):
@@ -60,12 +62,12 @@ def get_auto_format_func(
 
     Parameters
     ----------
-    col_name : str, optional
-        Column name (default=None)
+    col_name : object, optional
+        Hashable object to identify column like id or name. Default is None.
 
     possible_string_format_functions : func, optional
         Function that yields possible string formatting functions
-        (default=internal function to do this).
+        (defaults to internal function to do this).
 
     Returns
     -------
@@ -123,7 +125,8 @@ def get_auto_format_func(
                     # Does this string format method work?
                     out = format_func(format_, val)
                     # Require that the format statement actually did something.
-                    assert out != format_
+                    if out == format_:
+                        raise ValueError('the format passed in did nothing.')
                 except Exception:
                     continue
                 else:
@@ -207,7 +210,7 @@ class TableFormatter(object):
             Maximum lines of output (header + data rows)
 
         show_name : bool
-            Include column name (default=True)
+            Include column name. Default is True.
 
         show_unit : bool
             Include a header row for unit.  Default is to show a row
@@ -215,7 +218,7 @@ class TableFormatter(object):
             for the unit.
 
         show_dtype : bool
-            Include column dtype (default=False)
+            Include column dtype. Default is False.
 
         show_length : bool
             Include column length at end.  Default is to show this only
@@ -329,7 +332,7 @@ class TableFormatter(object):
             Maximum lines of output (header + data rows)
 
         show_name : bool
-            Include column name (default=True)
+            Include column name. Default is True.
 
         show_unit : bool
             Include a header row for unit.  Default is to show a row
@@ -341,7 +344,7 @@ class TableFormatter(object):
             defined within the iterator.
 
         show_dtype : bool
-            Include column dtype (default=False)
+            Include column dtype. Default is False.
 
         show_length : bool
             Include column length at end.  Default is to show this only
@@ -414,13 +417,13 @@ class TableFormatter(object):
         #    See Quantity for an example.
         #
         # - get_auto_format_func() returns a wrapped version of auto_format_func
-        #    with the column name and possible_string_format_functions as
+        #    with the column id and possible_string_format_functions as
         #    enclosed variables.
         col_format = col.info.format or getattr(col.info, 'default_format', None)
         pssf = (getattr(col.info, 'possible_string_format_functions', None) or
                 _possible_string_format_functions)
-        auto_format_func = get_auto_format_func(col.info.name, pssf)
-        format_key = (col_format, col.info.name)
+        auto_format_func = get_auto_format_func(id(col), pssf)
+        format_key = (col_format, id(col))
         format_func = _format_funcs.get(format_key, auto_format_func)
 
         if len(col) > max_lines:
@@ -472,7 +475,7 @@ class TableFormatter(object):
             Maximum character width of output
 
         show_name : bool
-            Include a header row for column names (default=True)
+            Include a header row for column names. Default is True.
 
         show_unit : bool
             Include a header row for unit.  Default is to show a row
@@ -480,10 +483,10 @@ class TableFormatter(object):
             for the unit.
 
         show_dtype : bool
-            Include a header row for column dtypes (default=False)
+            Include a header row for column dtypes. Default is False.
 
         html : bool
-            Format the output as an HTML table (default=False)
+            Format the output as an HTML table. Default is False.
 
         tableid : str or None
             An ID tag for the table; only used if html is set.  Default is
@@ -610,7 +613,7 @@ class TableFormatter(object):
             Maximum character width of output
 
         show_name : bool
-            Include a header row for column names (default=True)
+            Include a header row for column names. Default is True.
 
         show_unit : bool
             Include a header row for unit.  Default is to show a row
@@ -618,7 +621,7 @@ class TableFormatter(object):
             for the unit.
 
         show_dtype : bool
-            Include a header row for column dtypes (default=False)
+            Include a header row for column dtypes. Default is False.
         """
         allowed_keys = 'f br<>qhpn'
 
