@@ -2,15 +2,18 @@
 
 # TEST_UNICODE_LITERALS
 
+import pytest
 import numpy as np
 
-from ...tests.helper import pytest, catch_warnings
+from ...tests.helper import catch_warnings
 from ...table import Table, Column
 from ...utils.exceptions import AstropyUserWarning
 
+
 def sort_eq(list1, list2):
     return sorted(list1) == sorted(list2)
 
+
 def test_column_group_by(T1):
     for masked in (False, True):
         t1 = Table(T1, masked=masked)
@@ -28,6 +31,7 @@ def test_column_group_by(T1):
         t1ag = t1a.group_by(t1['a', 'b'].as_array())
         assert np.all(t1ag.groups.indices == np.array([0, 1, 3, 4, 5, 7, 8]))
 
+
 def test_table_group_by(T1):
     """
     Test basic table group_by functionality for possible key types and for
@@ -453,8 +457,10 @@ def test_table_aggregate_reduceat(T1):
     # Comparison functions without reduceat
     def np_mean(x):
         return np.mean(x)
+
     def np_sum(x):
         return np.sum(x)
+
     def np_add(x):
         return np.add(x)
 
