@@ -23,11 +23,12 @@ import os
 import re
 import sys
 import types
-import argparse
 from collections import OrderedDict
 
+import pytest
+
 from ..config.paths import set_temp_config, set_temp_cache
-from .helper import pytest, treat_deprecations_as_exceptions, ignore_warnings
+from .helper import treat_deprecations_as_exceptions, ignore_warnings
 from .helper import enable_deprecations_as_exceptions  # pylint: disable=W0611
 from .disable_internet import turn_off_internet, turn_on_internet
 from .output_checker import AstropyOutputChecker, FIX
@@ -123,6 +124,7 @@ def pytest_generate_tests(metafunc):
         metafunc.fixturenames.append('tmp_ct')
         metafunc.parametrize('tmp_ct', range(count))
 
+
 # We monkey-patch in our replacement doctest OutputChecker.  Not
 # great, but there isn't really an API to replace the checker when
 # using doctest.testfile, unfortunately.
@@ -146,8 +148,9 @@ def pytest_configure(config):
                           allow_astropy_data=config.getoption('remote_data') == 'astropy')
 
     doctest_plugin = config.pluginmanager.getplugin('doctest')
-    if (doctest_plugin is None or config.option.doctestmodules or not
-            (config.getini('doctest_plus') or config.option.doctest_plus)):
+    if (doctest_plugin is None or config.option.doctestmodules
+            or not (config.getini('doctest_plus').lower() in ['true', 'enabled']
+                    or config.option.doctest_plus)):
         return
 
     # These are the default doctest options we use for everything.
@@ -180,7 +183,8 @@ def pytest_configure(config):
 
             # uses internal doctest module parsing mechanism
             finder = DocTestFinderPlus()
-            runner = doctest.DebugRunner(verbose=False, optionflags=opts)
+            runner = doctest.DebugRunner(verbose=False, optionflags=opts,
+                                         checker=AstropyOutputChecker())
             for test in finder.find(module):
                 if test.examples:  # skip empty doctests
                     if config.getvalue("remote_data") != 'any':
@@ -191,21 +195,11 @@ def pytest_configure(config):
                     yield doctest_plugin.DoctestItem(
                         test.name, self, runner, test)
 
-        # This is for py.test prior to 2.4.0
-        def runtest(self):
-            return
-
-    class DocTestTextfilePlus(doctest_plugin.DoctestTextfile):
+    class DocTestTextfilePlus(doctest_plugin.DoctestItem, pytest.Module):
         def runtest(self):
             # satisfy `FixtureRequest` constructor...
             self.funcargs = {}
-            try:
-                self._fixtureinfo = doctest_plugin.FuncFixtureInfo((), [], {})
-                fixture_request = doctest_plugin.FixtureRequest(self)
-            except AttributeError:  # pytest >= 2.8.0
-                python_plugin = config.pluginmanager.getplugin('python')
-                self._fixtureinfo = python_plugin.FuncFixtureInfo((), [], {})
-                fixture_request = python_plugin.FixtureRequest(self)
+            fixture_request = doctest_plugin._setup_fixtures(self)
 
             failed, tot = doctest.testfile(
                 str(self.fspath), module_relative=False,
@@ -504,6 +498,7 @@ def pytest_runtest_setup(item):
             if source == 'any':
                 pytest.skip("need --remote-data option to run")
 
+
 def pytest_runtest_teardown(item, nextitem):
     if hasattr(item, 'set_temp_cache'):
         item.set_temp_cache.__exit__()
@@ -643,10 +638,13 @@ def pytest_report_header(config):
     special_opts = ["remote_data", "pep8"]
     opts = []
     for op in special_opts:
-        if getattr(config.option, op, None):
+        op_value = getattr(config.option, op, None)
+        if op_value:
+            if isinstance(op_value, six.string_types):
+                op = ': '.join((op, op_value))
             opts.append(op)
     if opts:
-        s += "Using Astropy options: {0}.\n".format(" ".join(opts))
+        s += "Using Astropy options: {0}.\n".format(", ".join(opts))
 
     if six.PY2:
         s = s.encode(stdoutencoding, 'replace')
@@ -670,6 +668,7 @@ class Pair(pytest.File):
     This class treats a given test .py file as a pair of .py files
     where one has __future__ unicode_literals and the other does not.
     """
+
     def collect(self):
         # First, just do the regular import of the module to make
         # sure it's sane and valid.  This block is copied directly
