@@ -4,13 +4,13 @@
 
 import copy
 import functools
-import sys
 import datetime
 from copy import deepcopy
 
+import pytest
 import numpy as np
 
-from ...tests.helper import pytest, catch_warnings
+from ...tests.helper import catch_warnings
 from ...tests.disable_internet import INTERNET_OFF
 from ...extern import six
 from ...extern.six.moves import zip
@@ -33,6 +33,7 @@ allclose_sec = functools.partial(np.allclose, rtol=2. ** -52,
 allclose_year = functools.partial(np.allclose, rtol=2. ** -52,
                                   atol=0.)  # 14 microsec at current epoch
 
+
 def setup_function(func):
     func.FORMATS_ORIG = deepcopy(Time.FORMATS)
 
@@ -93,21 +94,22 @@ class TestBasic():
         assert t4.isscalar is False
         assert t4.shape == np.broadcast(val, val2).shape
 
-    def test_copy_time(self):
+    @pytest.mark.parametrize('value', [2455197.5, [2455197.5]])
+    def test_copy_time(self, value):
         """Test copying the values of a Time object by passing it into the
         Time initializer.
         """
-        t = Time(2455197.5, format='jd', scale='utc')
+        t = Time(value, format='jd', scale='utc')
 
         t2 = Time(t, copy=False)
-        assert t.jd - t2.jd == 0
-        assert (t - t2).jd == 0
+        assert np.all(t.jd - t2.jd == 0)
+        assert np.all((t - t2).jd == 0)
         assert t._time.jd1 is t2._time.jd1
         assert t._time.jd2 is t2._time.jd2
 
         t2 = Time(t, copy=True)
-        assert t.jd - t2.jd == 0
-        assert (t - t2).jd == 0
+        assert np.all(t.jd - t2.jd == 0)
+        assert np.all((t - t2).jd == 0)
         assert t._time.jd1 is not t2._time.jd1
         assert t._time.jd2 is not t2._time.jd2
 
@@ -627,11 +629,20 @@ class TestSubFormat():
         # Test deprecated scale.
         t = Time('2000-01-02(IAT)')
         assert t.scale == 'tai'
+        # Test with scale and FITS string scale
+        t = Time('2045-11-08T00:00:00.000(UTC)', scale='utc')
+        assert t.scale == 'utc'
         # Check that inconsistent scales lead to errors.
         with pytest.raises(ValueError):
             Time('2000-01-02(TAI)', scale='utc')
         with pytest.raises(ValueError):
             Time(['2000-01-02(TAI)', '2001-02-03(UTC)'])
+        # Check that inconsistent FITS string scales lead to errors.
+        with pytest.raises(ValueError):
+            Time(['2000-01-02(TAI)', '2001-02-03(IAT)'])
+        # Check that inconsistent realizations lead to errors.
+        with pytest.raises(ValueError):
+            Time(['2000-01-02(ET(NIST))', '2001-02-03(ET)'])
 
     def test_fits_scale_representation(self):
         t = Time('1960-01-02T03:04:05.678(ET(NIST))')
@@ -821,13 +832,7 @@ def test_now():
     # times are more like microseconds.  But it seems safer in case some
     # platforms have slow clock calls or something.
 
-    # py < 2.7 doesn't have `total_seconds`
-    if sys.version_info[:2] < (2, 7):
-        total_secs = lambda td: (td.microseconds + (
-            td.seconds + td.days * 24 * 3600) * 10 ** 6) / 10 ** 6.
-    else:
-        total_secs = lambda td: td.total_seconds()
-    assert total_secs(dt) < 0.1
+    assert dt.total_seconds() < 0.1
 
 
 def test_decimalyear():
@@ -903,6 +908,7 @@ def test_TimeFormat_scale():
     t.unix
     assert t.unix == t.utc.unix
 
+
 def test_scale_conversion():
     if INTERNET_OFF:
         # With internet off (which is the default for testing) then this will
@@ -915,10 +921,9 @@ def test_scale_conversion():
         Time(Time.now().cxcsec, format='cxcsec', scale='ut1')
 
 
-
 def test_byteorder():
     """Ensure that bigendian and little-endian both work (closes #2942)"""
-    mjd = np.array([53000.00,54000.00])
+    mjd = np.array([53000.00, 54000.00])
     big_endian = mjd.astype('>f8')
     little_endian = mjd.astype('<f8')
     time_mjd = Time(mjd, format='mjd')
@@ -940,6 +945,7 @@ def test_datetime_tzinfo():
     t = Time(d)
     assert t.value == datetime.datetime(2002, 1, 2, 16, 3, 4)
 
+
 def test_subfmts_regex():
     """
     Test having a custom subfmts with a regular expression
@@ -953,6 +959,7 @@ def test_subfmts_regex():
     assert t.value == '+02000-02-03'
     assert t.jd == Time('2000-02-03').jd
 
+
 def test_set_format_basic():
     """
     Test basics of setting format attribute.
@@ -970,6 +977,7 @@ def test_set_format_basic():
         assert t._time.jd1 is t0._time.jd1
         assert t._time.jd2 is t0._time.jd2
 
+
 def test_set_format_shares_subfmt():
     """
     Set format and round trip through a format that shares out_subfmt
@@ -986,6 +994,7 @@ def test_set_format_shares_subfmt():
     assert t.value == tc.value
     assert t.precision == 5
 
+
 def test_set_format_does_not_share_subfmt():
     """
     Set format and round trip through a format that does not share out_subfmt
@@ -1000,6 +1009,7 @@ def test_set_format_does_not_share_subfmt():
     assert t.out_subfmt == '*'
     assert t.value == '2000-02-03T00:00:00.000(UTC)'  # date_hms
 
+
 def test_replicate_value_error():
     """
     Passing a bad format to replicate should raise ValueError, not KeyError.
@@ -1010,6 +1020,7 @@ def test_replicate_value_error():
         t1.replicate(format='definitely_not_a_valid_format')
     assert 'format must be one of' in str(err)
 
+
 def test_remove_astropy_time():
     """
     Make sure that 'astropy_time' format is really gone after #3857.  Kind of
@@ -1021,6 +1032,7 @@ def test_remove_astropy_time():
         Time(t1, format='astropy_time')
     assert 'format must be one of' in str(err)
 
+
 def test_isiterable():
     """
     Ensure that scalar `Time` instances are not reported as iterable by the
@@ -1036,6 +1048,7 @@ def test_isiterable():
               format='iso', scale='utc')
     assert isiterable(t2)
 
+
 def test_to_datetime():
     tz = TimezoneInfo(utc_offset=-10*u.hour, tzname='US/Hawaii')
     # The above lines produces a `datetime.tzinfo` object similar to:
@@ -1056,6 +1069,11 @@ def test_to_datetime():
         assert tz.tzname(dt) == tz_dt.tzname()
     assert np.all(time == forced_to_astropy_time)
 
+    with pytest.raises(ValueError) as e:
+        Time('2015-06-30 23:59:60.000').to_datetime()
+        assert 'does not support leap seconds' in str(e.message)
+
+
 @pytest.mark.skipif('not HAS_PYTZ')
 def test_to_datetime_pytz():
 
@@ -1076,6 +1094,7 @@ def test_to_datetime_pytz():
         assert tz.tzname(dt) == tz_dt.tzname()
     assert np.all(time == forced_to_astropy_time)
 
+
 def test_cache():
     t = Time('2010-09-03 00:00:00')
     t2 = Time('2010-09-03 00:00:00')
