@@ -33,13 +33,21 @@ class Fits(generic.Generic):
         names = {}
         deprecated_names = set()
 
+        # Note about deprecated units: before v2.0, several units were treated
+        # as deprecated (G, barn, erg, Angstrom, angstrom). However, in the
+        # FITS 3.0 standard, these units are explicitly listed in the allowed
+        # units, but deprecated in the IAU Style Manual (McNally 1988). So
+        # after discussion (https://github.com/astropy/astropy/issues/2933),
+        # these units have been removed from the lists of deprecated units and
+        # bases.
+
         bases = [
             'm', 'g', 's', 'rad', 'sr', 'K', 'A', 'mol', 'cd',
             'Hz', 'J', 'W', 'V', 'N', 'Pa', 'C', 'Ohm', 'S',
             'F', 'Wb', 'T', 'H', 'lm', 'lx', 'a', 'yr', 'eV',
-            'pc', 'Jy', 'mag', 'R', 'bit', 'byte'
+            'pc', 'Jy', 'mag', 'R', 'bit', 'byte', 'G', 'barn'
         ]
-        deprecated_bases = ['G', 'barn']
+        deprecated_bases = []
         prefixes = [
             'y', 'z', 'a', 'f', 'p', 'n', 'u', 'm', 'c', 'd',
             '', 'da', 'h', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y']
@@ -63,9 +71,9 @@ class Fits(generic.Generic):
             'deg', 'arcmin', 'arcsec', 'mas', 'min', 'h', 'd', 'Ry',
             'solMass', 'u', 'solLum', 'solRad', 'AU', 'lyr', 'count',
             'ct', 'photon', 'ph', 'pixel', 'pix', 'D', 'Sun', 'chan',
-            'bin', 'voxel', 'adu', 'beam'
+            'bin', 'voxel', 'adu', 'beam', 'erg', 'Angstrom', 'angstrom'
         ]
-        deprecated_units = ['erg', 'Angstrom', 'angstrom']
+        deprecated_units = []
 
         for unit in simple_units + deprecated_units:
             names[unit] = getattr(u, unit)
