@@ -6,12 +6,14 @@ import functools
 import inspect
 import pickle
 
+import pytest
+
 from ..decorators import (deprecated_attribute, deprecated, wraps,
                           sharedmethod, classproperty,
                           format_doc, deprecated_renamed_argument)
 from ..exceptions import AstropyDeprecationWarning, AstropyUserWarning
 from ...extern import six
-from ...tests.helper import pytest, catch_warnings
+from ...tests.helper import catch_warnings
 
 
 def test_wraps():
@@ -135,6 +137,7 @@ class TA(object):
     """
     This is the class docstring.
     """
+
     def __init__(self):
         """
         This is the __init__ docstring
@@ -354,6 +357,35 @@ def test_deprecated_argument_relaxed():
         assert len(w) == 1
 
 
+def test_deprecated_argument_pending():
+    # Relax turns the TypeError if both old and new keyword are used into
+    # a warning.
+    @deprecated_renamed_argument('clobber', 'overwrite', '1.3', pending=True)
+    def test(overwrite):
+        return overwrite
+
+    # As positional argument only
+    assert test(1) == 1
+
+    # As new keyword argument
+    assert test(overwrite=1) == 1
+
+    # Using the deprecated name
+    with catch_warnings(AstropyUserWarning, AstropyDeprecationWarning) as w:
+        assert test(clobber=1) == 1
+        assert len(w) == 0
+
+    # Using both. Both keyword
+    with catch_warnings(AstropyUserWarning, AstropyDeprecationWarning) as w:
+        assert test(clobber=2, overwrite=1) == 1
+        assert len(w) == 0
+
+    # One positional, one keyword
+    with catch_warnings(AstropyUserWarning, AstropyDeprecationWarning) as w:
+        assert test(1, clobber=2) == 1
+        assert len(w) == 0
+
+
 def test_deprecated_argument_multi_deprecation():
     @deprecated_renamed_argument(['x', 'y', 'z'], ['a', 'b', 'c'],
                                  [1.3, 1.2, 1.3], relax=True)
@@ -482,6 +514,7 @@ def test_sharedmethod_reuse_on_subclasses():
             return cls.x
 
     six.add_metaclass(AMeta)
+
     class A(object):
         x = 3
 
@@ -710,7 +743,7 @@ def test_format_doc_onMethod():
     assert inspect.getdoc(TestClass.test_method) == 'what we do is strange.'
 
 
-#@pytest.mark.skipif('six.PY2')
+# @pytest.mark.skipif('six.PY2')
 def test_format_doc_onClass():
     # Check if the decorator works on classes too
     docstring = 'what we do {__doc__} {0}{opt}'
